/*
 * SaferoomMidlet.java
 *
 * Copyright (C) 2005 Tommi Laukkanen
 * http://www.substanceofcode.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.substanceofcode.saferoom.view;

import com.substanceofcode.saferoom.control.*;
import com.substanceofcode.saferoom.data.PasswordGroup;
import com.substanceofcode.saferoom.data.PasswordGroupCollection;
import com.substanceofcode.saferoom.data.PasswordItem;
import com.substanceofcode.saferoom.data.PasswordItemCollection;
import com.substanceofcode.utils.Settings;

import java.io.IOException;
import java.util.Enumeration;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.RecordStoreException;
import org.netbeans.microedition.util.CancellableTask;

/**
 *
 * @author Tommi Laukkanen
 */
public class SaferoomMidlet extends MIDlet implements javax.microedition.lcdui.CommandListener {
    
    /** Saferoom database */
    private static SaferoomController m_controller = null;
    private boolean m_addingNewGroup = false;
    private boolean m_addingNewPassword = false;
    private PasswordGroup m_currentGroup = null;
    private PasswordItem m_currentPassword = null;
    private CancellableTask m_openTask = null;
    private PasswordGroupCollection m_passwords = null;
    
    /** Creates a new instance of SaferoomMidlet */
    public SaferoomMidlet() {
        initialize();
        try {
            m_controller = SaferoomController.getInstance( this );
        } catch (RecordStoreException ex) {
            System.out.println("RecordStoreException while opening up the " +
                    "record store: " + ex.toString());
            ex.printStackTrace();
        } catch (IOException ex) {
            System.out.println("IOException while opening up the record store: "
                    + ex.toString());
            ex.printStackTrace();
        }
    }
    
    /** Login to database */
    public void login() {
        m_controller.openDatabase(m_passwordTextField.getString());
    }
    
    /** Update group list */
    private void updateGroupList() {
        get_groupList().deleteAll();
        String[] groups = m_controller.getGroups();
        for(int index=0; index<groups.length; index++) {
            get_groupList().append(groups[index], null);
        }
    }
    
    /** Update passwords list */
    private void updatePasswordsList() {
        get_passwordsList().deleteAll();
        PasswordItemCollection passwords = m_currentGroup.getPasswords();
        Enumeration passwordItemEnumeration = passwords.elements();
        while(passwordItemEnumeration.hasMoreElements()) {
            PasswordItem password = (PasswordItem) passwordItemEnumeration.nextElement();
            String passwordLocation = password.getLocation();
            get_passwordsList().append( passwordLocation, null);
        }
    }    
    
    /** Update password properties */
    private void updatePasswordProperties() {
        get_m_locationTextField().setString( m_currentPassword.getLocation() );
        get_m_passwordItemTextField().setString( m_currentPassword.getPassword() );
        get_m_usernameTextField().setString( m_currentPassword.getUsername() );
        get_m_descriptionTextField().setString( m_currentPassword.getComment() );        
    }
    
    private org.netbeans.microedition.lcdui.SplashScreen splashScreen;//GEN-BEGIN:MVDFields
    private Image m_splashImage;
    private List groupList;
    private Command m_openGroupCmd;
    private Command m_AddGroupCmd;
    private Command m_removeGroupCmd;
    private Command m_aboutCmd;
    private Command m_exitGroupCmd;
    private org.netbeans.microedition.lcdui.WaitScreen openDatabaseWaitScreen;
    private Form mainPasswordForm;
    private Command m_exitLoginCmd;
    private Command m_loginCmd;
    private Alert wrongPasswordAlert;
    private Command m_backAboutCmd;
    private Form groupPropertiesForm;
    private List passwordsList;
    private Command m_backPasswordCmd;
    private Command m_addPasswordCmd;
    private Form passwordPropertiesForm;
    private Command m_editPasswordCmd;
    private Command m_okPasswordCmd;
    private Command m_removePasswordCmd;
    private Command m_okGroupCmd;
    private Command exitCommand2;
    private Command m_editGroupCmd;
    private TextField m_passwordTextField;
    private Image m_lockImage;
    private Form aboutForm;
    private Command backCommand1;
    private StringItem titleStringItem;
    private StringItem licenseStringItem;
    private TextField m_groupNameTextField;
    private TextField m_locationTextField;
    private TextField m_usernameTextField;
    private TextField m_passwordItemTextField;
    private TextField m_descriptionTextField;
    private Command cancelCommand1;
    private org.netbeans.microedition.util.SimpleCancellableTask simpleCancellableTask;
    private Command m_changePasswordCmd;
    private TextBox newPasswordTextBox;
    private Command okCommand1;
    private Command cancelCommand2;
    private Command cancelCommand3;//GEN-END:MVDFields
    
//GEN-LINE:MVDMethods
    
    /** This method initializes UI of the application.//GEN-BEGIN:MVDInitBegin
     */
    private void initialize() {//GEN-END:MVDInitBegin
        // Insert pre-init code here
        getDisplay().setCurrent(get_splashScreen());//GEN-LINE:MVDInitInit
        // Insert post-init code here
    }//GEN-LINE:MVDInitEnd
    
    /** Called by the system to indicate that a command has been invoked on a particular displayable.//GEN-BEGIN:MVDCABegin
     * @param command the Command that ws invoked
     * @param displayable the Displayable on which the command was invoked
     */
    public void commandAction(Command command, Displayable displayable) {//GEN-END:MVDCABegin
        // Insert global pre-action code here
        if (displayable == mainPasswordForm) {//GEN-BEGIN:MVDCABody
            if (command == m_exitLoginCmd) {//GEN-END:MVDCABody
                // Insert pre-action code here
                exitMIDlet();//GEN-LINE:MVDCAAction25
                // Insert post-action code here
            } else if (command == m_loginCmd) {//GEN-LINE:MVDCACase25
                Settings settings=null;
                try {
                    settings = Settings.getInstance(this);
                } catch (IOException ex) {
                    System.err.println("IO error while getting settings.");
                    ex.printStackTrace();
                } catch (RecordStoreException ex) {
                    System.err.println("RecordStoreException while getting settings."); 
                    ex.printStackTrace();
                }
                String repository = settings.getStringProperty("repository","");
                String givenPassword = m_passwordTextField.getString();
                if(repository.length()>0) {
                    boolean passwordIsOk = m_controller.openDatabase( givenPassword );
                    if( passwordIsOk==true ) {
                        System.out.println("password=true");
                        getDisplay().setCurrent(get_groupList());
                        System.out.println("Grouplist set as current");
                        return;
                    } else {
                        System.out.println("password=false");
                        m_passwordTextField.setString("");
                        getDisplay().setCurrent(get_wrongPasswordAlert());
                        return;
                    }
                } else {
                    // First time
                    getDisplay().setCurrent(get_groupList());
                    m_controller.setPassword(givenPassword);
                    settings.setStringProperty("Password", givenPassword);
                }
                //m_openTask = new SaferoomRepositoryOpenerTask(passwordIsOk);
                /*
                get_openDatabaseWaitScreen().setNextDisplayable(get_groupList());
getDisplay ().setCurrent (get_openDatabaseWaitScreen());//GEN-LINE:MVDCAAction27
                */
            }//GEN-BEGIN:MVDCACase27
        } else if (displayable == groupList) {
            if (command == m_removeGroupCmd) {//GEN-END:MVDCACase27
                // Insert pre-action code here
                
                // Delete selected group and passwords
                int selectedIndex = get_groupList().getSelectedIndex();
                String selectedGroupName = get_groupList().getString( selectedIndex );
                PasswordGroup group = m_controller.getGroup(selectedGroupName);
                m_controller.deleteGroup(group);
                
                updateGroupList();
                
                getDisplay().setCurrent(get_groupList());//GEN-LINE:MVDCAAction15
                // Insert post-action code here
            } else if (command == m_exitGroupCmd) {//GEN-LINE:MVDCACase15
                
                // Save repository
                m_controller.closeDatabase();
                
                exitMIDlet();//GEN-LINE:MVDCAAction19
                // Insert post-action code here
            } else if (command == m_aboutCmd) {//GEN-LINE:MVDCACase19
                // Insert pre-action code here
                getDisplay().setCurrent(get_aboutForm());//GEN-LINE:MVDCAAction17
                // Insert post-action code here
            } else if (command == m_AddGroupCmd) {//GEN-LINE:MVDCACase17
                // Insert pre-action code here
                getDisplay().setCurrent(get_groupPropertiesForm());//GEN-LINE:MVDCAAction13
                m_addingNewGroup = true;
                m_groupNameTextField.setString("");
            } else if (command == m_openGroupCmd) {//GEN-LINE:MVDCACase13
                // Insert pre-action code here
                
                int selectedIndex = get_groupList().getSelectedIndex();
                String selectedGroupName = get_groupList().getString( selectedIndex );
                PasswordGroup group = m_controller.getGroup(selectedGroupName);
                if( group==null ) {
                    System.err.println("Unable to find selected password group");
                    return;
                }
                m_currentGroup = group;
                updatePasswordsList();
                
                getDisplay().setCurrent(get_passwordsList());//GEN-LINE:MVDCAAction11
                // Insert post-action code here
            } else if (command == m_editGroupCmd) {//GEN-LINE:MVDCACase11
                // Insert pre-action code here
                m_addingNewGroup = false;
                
                int selectedIndex = get_groupList().getSelectedIndex();
                String selectedGroupName = get_groupList().getString( selectedIndex );
                PasswordGroup group = m_controller.getGroup(selectedGroupName);
                if( group==null ) {
                    System.err.println("Unable to find selected password group");
                    return;
                }
                m_currentGroup = group;                
                get_groupPropertiesForm();
                m_groupNameTextField.setString(m_currentGroup.getName());
                getDisplay().setCurrent(get_groupPropertiesForm());//GEN-LINE:MVDCAAction56
                // Insert post-action code here
                
            } else if (command == m_changePasswordCmd) {//GEN-LINE:MVDCACase56
                // Insert pre-action code here
                getDisplay().setCurrent(get_newPasswordTextBox());//GEN-LINE:MVDCAAction80
                // Insert post-action code here
            }//GEN-BEGIN:MVDCACase80
        } else if (displayable == passwordsList) {
            if (command == m_removePasswordCmd) {//GEN-END:MVDCACase80
                // Insert pre-action code here
                
                int selectedIndex = get_passwordsList().getSelectedIndex();
                String selectedPasswordLocation = get_passwordsList().getString( selectedIndex );
                m_passwords =  m_controller.removePassword(m_currentGroup, selectedPasswordLocation);
                
                updatePasswordsList();
                
                getDisplay().setCurrent(get_passwordsList());//GEN-LINE:MVDCAAction47
                // Insert post-action code here
            } else if (command == m_editPasswordCmd) {//GEN-LINE:MVDCACase47
                // Insert pre-action code here
                m_addingNewPassword = false;
                
                // Set password form properties
                int selectedIndex = get_passwordsList().getSelectedIndex();
                String selectedPasswordLocation = get_passwordsList().getString( selectedIndex );
                PasswordItem password = m_controller.getPassword( 
                        m_currentGroup, 
                        selectedPasswordLocation );
                if( password==null ) {
                    System.err.println("Unable to find selected password item");
                    return;
                }
                m_currentPassword = password;
                
                updatePasswordProperties();
                
                getDisplay().setCurrent(get_passwordPropertiesForm());//GEN-LINE:MVDCAAction43
                // Insert post-action code here
            } else if (command == m_backPasswordCmd) {//GEN-LINE:MVDCACase43
                // Insert pre-action code here
                getDisplay().setCurrent(get_groupList());//GEN-LINE:MVDCAAction38
                // Insert post-action code here
            } else if (command == m_addPasswordCmd) {//GEN-LINE:MVDCACase38
                // Insert pre-action code here
                m_addingNewPassword = true;
                get_m_locationTextField().setString("");
                get_m_usernameTextField().setString("");
                get_m_passwordItemTextField().setString("");
                get_m_descriptionTextField().setString("");
                getDisplay().setCurrent(get_passwordPropertiesForm());//GEN-LINE:MVDCAAction40
                // Insert post-action code here
            }//GEN-BEGIN:MVDCACase40
        } else if (displayable == groupPropertiesForm) {
            if (command == m_okGroupCmd) {//GEN-END:MVDCACase40
                
                if( m_addingNewGroup==true )
                {
                    // adding new group
                    PasswordGroup group = new PasswordGroup( m_groupNameTextField.getString() );
                    m_controller.addGroup( group );
                } else {
                    // update existing group
                    int selectedIndex = get_groupList().getSelectedIndex();
                    String selectedGroupName = get_groupList().getString( selectedIndex );
                    PasswordGroup group = m_controller.getGroup(selectedGroupName);
                    if( group==null ) {
                        System.err.println("Unable to find selected password group");
                        return;
                    }
                    PasswordGroup oldGroup = group;
                    m_controller.deleteGroup(oldGroup);

                    PasswordGroup newGroup = group;
                    String newGroupName;
                    newGroupName = m_groupNameTextField.getString();
                    newGroup.setName(newGroupName);
                    m_controller.addGroup(newGroup);
                }
                
                updateGroupList();            
                
                getDisplay().setCurrent(get_groupList());//GEN-LINE:MVDCAAction52
                // Insert post-action code here
            } else if (command == cancelCommand3) {//GEN-LINE:MVDCACase52
                // Insert pre-action code here
                getDisplay().setCurrent(get_groupList());//GEN-LINE:MVDCAAction87
                // Insert post-action code here
            }//GEN-BEGIN:MVDCACase87
        } else if (displayable == passwordPropertiesForm) {
            if (command == m_okPasswordCmd) {//GEN-END:MVDCACase87

                
                PasswordItem password = new PasswordItem( 
                        m_locationTextField.getString(),
                        m_usernameTextField.getString(),
                        m_passwordItemTextField.getString(),
                        m_descriptionTextField.getString());
                if(m_addingNewPassword==false) {
                    m_controller.removePassword(
                            m_currentGroup, 
                            m_currentPassword.getLocation());
                }
                m_passwords = m_controller.addPassword(
                        password, 
                        m_currentGroup);

                updatePasswordsList();
                
                getDisplay().setCurrent(get_passwordsList());//GEN-LINE:MVDCAAction45
                // Insert post-action code here
            }//GEN-BEGIN:MVDCACase45
        } else if (displayable == aboutForm) {
            if (command == backCommand1) {//GEN-END:MVDCACase45
                // Insert pre-action code here
                getDisplay().setCurrent(get_groupList());//GEN-LINE:MVDCAAction66
                // Insert post-action code here
            }//GEN-BEGIN:MVDCACase66
        } else if (displayable == newPasswordTextBox) {
            if (command == cancelCommand2) {//GEN-END:MVDCACase66
                // Insert pre-action code here
                getDisplay().setCurrent(get_groupList());//GEN-LINE:MVDCAAction85
                // Insert post-action code here
            } else if (command == okCommand1) {//GEN-LINE:MVDCACase85
                // Insert pre-action code here
                
                // Change the password
                String newPassword;
                newPassword = newPasswordTextBox.getString();
                m_controller.setPassword(newPassword);                
                
                getDisplay().setCurrent(get_groupList());//GEN-LINE:MVDCAAction83
                // Insert post-action code here
            }//GEN-BEGIN:MVDCACase83
        }//GEN-END:MVDCACase83
        // Insert global post-action code here
}//GEN-LINE:MVDCAEnd
    
    /**
     * This method should return an instance of the display.
     */
    public javax.microedition.lcdui.Display getDisplay () {//GEN-FIRST:MVDGetDisplay
        return javax.microedition.lcdui.Display.getDisplay(this);
    }//GEN-LAST:MVDGetDisplay
    
    /**
     * This method should exit the midlet.
     */
    public void exitMIDlet () {//GEN-FIRST:MVDExitMidlet
        getDisplay().setCurrent(null);
        destroyApp(true);
        notifyDestroyed();
    }//GEN-LAST:MVDExitMidlet
    
    /** This method returns instance for splashScreen component and should be called instead of accessing splashScreen field directly.//GEN-BEGIN:MVDGetBegin2
     * @return Instance for splashScreen component
     */
    public org.netbeans.microedition.lcdui.SplashScreen get_splashScreen() {
        if (splashScreen == null) {//GEN-END:MVDGetBegin2
            // Insert pre-init code here
            splashScreen = new org.netbeans.microedition.lcdui.SplashScreen(getDisplay());//GEN-BEGIN:MVDGetInit2
            splashScreen.setImage(get_m_splashImage());
            splashScreen.setNextDisplayable(get_mainPasswordForm());//GEN-END:MVDGetInit2
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd2
        return splashScreen;
    }//GEN-END:MVDGetEnd2
    
    /** This method returns instance for m_splashImage component and should be called instead of accessing m_splashImage field directly.//GEN-BEGIN:MVDGetBegin4
     * @return Instance for m_splashImage component
     */
    public Image get_m_splashImage() {
        if (m_splashImage == null) {//GEN-END:MVDGetBegin4
            // Insert pre-init code here
            try {//GEN-BEGIN:MVDGetInit4
                m_splashImage = Image.createImage("/icons/splash.png");
            } catch (java.io.IOException exception) {
            }//GEN-END:MVDGetInit4
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd4
        return m_splashImage;
    }//GEN-END:MVDGetEnd4
    
    /** This method returns instance for groupList component and should be called instead of accessing groupList field directly.//GEN-BEGIN:MVDGetBegin8
     * @return Instance for groupList component
     */
    public List get_groupList() {
        if (groupList == null) {//GEN-END:MVDGetBegin8
            // Insert pre-init code here
            groupList = new List("Groups", Choice.IMPLICIT, new String[0], new Image[0]);//GEN-BEGIN:MVDGetInit8
            groupList.addCommand(get_m_openGroupCmd());
            groupList.addCommand(get_m_AddGroupCmd());
            groupList.addCommand(get_m_removeGroupCmd());
            groupList.addCommand(get_m_aboutCmd());
            groupList.addCommand(get_m_exitGroupCmd());
            groupList.addCommand(get_m_editGroupCmd());
            groupList.addCommand(get_m_changePasswordCmd());
            groupList.setCommandListener(this);
            groupList.setSelectedFlags(new boolean[0]);
            groupList.setSelectCommand(get_m_openGroupCmd());//GEN-END:MVDGetInit8
            
            String[] groups = m_controller.getGroups();
            groupList.deleteAll();
            if( groups != null) {
                for(int index=0; index<groups.length; index++) {
                    groupList.append( groups[ index ], null );
                }
            }
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd8
        return groupList;
    }//GEN-END:MVDGetEnd8
    
    /** This method returns instance for m_openGroupCmd component and should be called instead of accessing m_openGroupCmd field directly.//GEN-BEGIN:MVDGetBegin10
     * @return Instance for m_openGroupCmd component
     */
    public Command get_m_openGroupCmd() {
        if (m_openGroupCmd == null) {//GEN-END:MVDGetBegin10
            // Insert pre-init code here
            m_openGroupCmd = new Command("Open", Command.ITEM, 1);//GEN-LINE:MVDGetInit10
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd10
        return m_openGroupCmd;
    }//GEN-END:MVDGetEnd10
    
    /** This method returns instance for m_AddGroupCmd component and should be called instead of accessing m_AddGroupCmd field directly.//GEN-BEGIN:MVDGetBegin12
     * @return Instance for m_AddGroupCmd component
     */
    public Command get_m_AddGroupCmd() {
        if (m_AddGroupCmd == null) {//GEN-END:MVDGetBegin12
            // Insert pre-init code here
            m_AddGroupCmd = new Command("Add", Command.SCREEN, 2);//GEN-LINE:MVDGetInit12
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd12
        return m_AddGroupCmd;
    }//GEN-END:MVDGetEnd12
    
    /** This method returns instance for m_removeGroupCmd component and should be called instead of accessing m_removeGroupCmd field directly.//GEN-BEGIN:MVDGetBegin14
     * @return Instance for m_removeGroupCmd component
     */
    public Command get_m_removeGroupCmd() {
        if (m_removeGroupCmd == null) {//GEN-END:MVDGetBegin14
            // Insert pre-init code here
            m_removeGroupCmd = new Command("Remove", Command.SCREEN, 4);//GEN-LINE:MVDGetInit14
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd14
        return m_removeGroupCmd;
    }//GEN-END:MVDGetEnd14
    
    /** This method returns instance for m_aboutCmd component and should be called instead of accessing m_aboutCmd field directly.//GEN-BEGIN:MVDGetBegin16
     * @return Instance for m_aboutCmd component
     */
    public Command get_m_aboutCmd() {
        if (m_aboutCmd == null) {//GEN-END:MVDGetBegin16
            // Insert pre-init code here
            m_aboutCmd = new Command("About", Command.SCREEN, 5);//GEN-LINE:MVDGetInit16
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd16
        return m_aboutCmd;
    }//GEN-END:MVDGetEnd16
    
    /** This method returns instance for m_exitGroupCmd component and should be called instead of accessing m_exitGroupCmd field directly.//GEN-BEGIN:MVDGetBegin18
     * @return Instance for m_exitGroupCmd component
     */
    public Command get_m_exitGroupCmd() {
        if (m_exitGroupCmd == null) {//GEN-END:MVDGetBegin18
            // Insert pre-init code here
            m_exitGroupCmd = new Command("Exit", Command.EXIT, 7);//GEN-LINE:MVDGetInit18
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd18
        return m_exitGroupCmd;
    }//GEN-END:MVDGetEnd18
    
    /** This method returns instance for openDatabaseWaitScreen component and should be called instead of accessing openDatabaseWaitScreen field directly.//GEN-BEGIN:MVDGetBegin20
     * @return Instance for openDatabaseWaitScreen component
     */
    public org.netbeans.microedition.lcdui.WaitScreen get_openDatabaseWaitScreen() {
        if (openDatabaseWaitScreen == null) {//GEN-END:MVDGetBegin20
            // Insert pre-init code here
            openDatabaseWaitScreen = new org.netbeans.microedition.lcdui.WaitScreen(getDisplay());//GEN-BEGIN:MVDGetInit20
            openDatabaseWaitScreen.setFailureDisplayable(get_wrongPasswordAlert(), get_mainPasswordForm());
            openDatabaseWaitScreen.setTitle("Please wait");
            openDatabaseWaitScreen.setText("Opening Saferoom Repository");
            openDatabaseWaitScreen.setNextDisplayable(get_groupList());
            openDatabaseWaitScreen.setTask(get_simpleCancellableTask());//GEN-END:MVDGetInit20
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd20
        return openDatabaseWaitScreen;
    }//GEN-END:MVDGetEnd20
    
    /** This method returns instance for mainPasswordForm component and should be called instead of accessing mainPasswordForm field directly.//GEN-BEGIN:MVDGetBegin23
     * @return Instance for mainPasswordForm component
     */
    public Form get_mainPasswordForm() {
        if (mainPasswordForm == null) {//GEN-END:MVDGetBegin23
            Settings settings=null;
            try {
                settings = Settings.getInstance(this);
                String data = settings.getStringProperty("repository", "");
                if(data.length()==0) {
                    get_m_passwordTextField().setLabel("Enter new password for repository");
                } else {
                    get_m_passwordTextField().setLabel("Enter password");
                }  
            } catch (RecordStoreException ex) {
                ex.printStackTrace();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            mainPasswordForm = new Form("Login", new Item[] {get_m_passwordTextField()});//GEN-BEGIN:MVDGetInit23
            mainPasswordForm.addCommand(get_m_exitLoginCmd());
            mainPasswordForm.addCommand(get_m_loginCmd());
            mainPasswordForm.setCommandListener(this);//GEN-END:MVDGetInit23
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd23
        return mainPasswordForm;
    }//GEN-END:MVDGetEnd23
    
    /** This method returns instance for m_exitLoginCmd component and should be called instead of accessing m_exitLoginCmd field directly.//GEN-BEGIN:MVDGetBegin24
     * @return Instance for m_exitLoginCmd component
     */
    public Command get_m_exitLoginCmd() {
        if (m_exitLoginCmd == null) {//GEN-END:MVDGetBegin24
            // Insert pre-init code here
            m_exitLoginCmd = new Command("Exit", Command.EXIT, 1);//GEN-LINE:MVDGetInit24
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd24
        return m_exitLoginCmd;
    }//GEN-END:MVDGetEnd24
    
    /** This method returns instance for m_loginCmd component and should be called instead of accessing m_loginCmd field directly.//GEN-BEGIN:MVDGetBegin26
     * @return Instance for m_loginCmd component
     */
    public Command get_m_loginCmd() {
        if (m_loginCmd == null) {//GEN-END:MVDGetBegin26
            // Insert pre-init code here
            m_loginCmd = new Command("Login", Command.ITEM, 1);//GEN-LINE:MVDGetInit26
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd26
        return m_loginCmd;
    }//GEN-END:MVDGetEnd26
    
    /** This method returns instance for wrongPasswordAlert component and should be called instead of accessing wrongPasswordAlert field directly.//GEN-BEGIN:MVDGetBegin28
     * @return Instance for wrongPasswordAlert component
     */
    public Alert get_wrongPasswordAlert() {
        if (wrongPasswordAlert == null) {//GEN-END:MVDGetBegin28
            // Insert pre-init code here
            wrongPasswordAlert = new Alert("Access denied", "You have entered an invalid password. Please check the entered password!", null, AlertType.INFO);//GEN-BEGIN:MVDGetInit28
            wrongPasswordAlert.setTimeout(-2);//GEN-END:MVDGetInit28
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd28
        return wrongPasswordAlert;
    }//GEN-END:MVDGetEnd28
    
    /** This method returns instance for m_backAboutCmd component and should be called instead of accessing m_backAboutCmd field directly.//GEN-BEGIN:MVDGetBegin30
     * @return Instance for m_backAboutCmd component
     */
    public Command get_m_backAboutCmd() {
        if (m_backAboutCmd == null) {//GEN-END:MVDGetBegin30
            // Insert pre-init code here
            m_backAboutCmd = new Command("Back", Command.BACK, 1);//GEN-LINE:MVDGetInit30
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd30
        return m_backAboutCmd;
    }//GEN-END:MVDGetEnd30
    /** This method returns instance for groupPropertiesForm component and should be called instead of accessing groupPropertiesForm field directly.//GEN-BEGIN:MVDGetBegin34
     * @return Instance for groupPropertiesForm component
     */
    public Form get_groupPropertiesForm() {
        if (groupPropertiesForm == null) {//GEN-END:MVDGetBegin34
            // Insert pre-init code here
            groupPropertiesForm = new Form("Properties", new Item[] {get_m_groupNameTextField()});//GEN-BEGIN:MVDGetInit34
            groupPropertiesForm.addCommand(get_m_okGroupCmd());
            groupPropertiesForm.addCommand(get_cancelCommand3());
            groupPropertiesForm.setCommandListener(this);//GEN-END:MVDGetInit34
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd34
        return groupPropertiesForm;
    }//GEN-END:MVDGetEnd34
    
    /** This method returns instance for passwordsList component and should be called instead of accessing passwordsList field directly.//GEN-BEGIN:MVDGetBegin35
     * @return Instance for passwordsList component
     */
    public List get_passwordsList() {
        if (passwordsList == null) {//GEN-END:MVDGetBegin35
            // Insert pre-init code here
            passwordsList = new List("Passwords", Choice.IMPLICIT, new String[0], new Image[0]);//GEN-BEGIN:MVDGetInit35
            passwordsList.addCommand(get_m_backPasswordCmd());
            passwordsList.addCommand(get_m_addPasswordCmd());
            passwordsList.addCommand(get_m_editPasswordCmd());
            passwordsList.addCommand(get_m_removePasswordCmd());
            passwordsList.setCommandListener(this);
            passwordsList.setSelectedFlags(new boolean[0]);
            passwordsList.setSelectCommand(get_m_editPasswordCmd());//GEN-END:MVDGetInit35
            // Insert post-init code here
            PasswordItemCollection passwords = m_currentGroup.getPasswords();
            passwordsList.deleteAll();
            
            if( passwords != null) {
                Enumeration passwordEnumeration  = passwords.elements();
                while(passwordEnumeration.hasMoreElements()) {
                    PasswordItem password = (PasswordItem)passwordEnumeration.nextElement();
                    String location = password.getLocation();
                    passwordsList.append( location, null);
                }
            }
            
        }//GEN-BEGIN:MVDGetEnd35
        return passwordsList;
    }//GEN-END:MVDGetEnd35
    
    /** This method returns instance for m_backPasswordCmd component and should be called instead of accessing m_backPasswordCmd field directly.//GEN-BEGIN:MVDGetBegin37
     * @return Instance for m_backPasswordCmd component
     */
    public Command get_m_backPasswordCmd() {
        if (m_backPasswordCmd == null) {//GEN-END:MVDGetBegin37
            // Insert pre-init code here
            m_backPasswordCmd = new Command("Back", Command.BACK, 1);//GEN-LINE:MVDGetInit37
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd37
        return m_backPasswordCmd;
    }//GEN-END:MVDGetEnd37
    
    /** This method returns instance for m_addPasswordCmd component and should be called instead of accessing m_addPasswordCmd field directly.//GEN-BEGIN:MVDGetBegin39
     * @return Instance for m_addPasswordCmd component
     */
    public Command get_m_addPasswordCmd() {
        if (m_addPasswordCmd == null) {//GEN-END:MVDGetBegin39
            // Insert pre-init code here
            m_addPasswordCmd = new Command("Add", Command.SCREEN, 1);//GEN-LINE:MVDGetInit39
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd39
        return m_addPasswordCmd;
    }//GEN-END:MVDGetEnd39
    
    /** This method returns instance for passwordPropertiesForm component and should be called instead of accessing passwordPropertiesForm field directly.//GEN-BEGIN:MVDGetBegin41
     * @return Instance for passwordPropertiesForm component
     */
    public Form get_passwordPropertiesForm() {
        if (passwordPropertiesForm == null) {//GEN-END:MVDGetBegin41
            // Insert pre-init code here
            passwordPropertiesForm = new Form("Credentials", new Item[] {//GEN-BEGIN:MVDGetInit41
                get_m_locationTextField(),
                get_m_usernameTextField(),
                get_m_passwordItemTextField(),
                get_m_descriptionTextField()
            });
            passwordPropertiesForm.addCommand(get_m_okPasswordCmd());
            passwordPropertiesForm.setCommandListener(this);//GEN-END:MVDGetInit41
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd41
        return passwordPropertiesForm;
    }//GEN-END:MVDGetEnd41
    
    /** This method returns instance for m_editPasswordCmd component and should be called instead of accessing m_editPasswordCmd field directly.//GEN-BEGIN:MVDGetBegin42
     * @return Instance for m_editPasswordCmd component
     */
    public Command get_m_editPasswordCmd() {
        if (m_editPasswordCmd == null) {//GEN-END:MVDGetBegin42
            // Insert pre-init code here
            m_editPasswordCmd = new Command("Edit", Command.ITEM, 1);//GEN-LINE:MVDGetInit42
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd42
        return m_editPasswordCmd;
    }//GEN-END:MVDGetEnd42
    
    /** This method returns instance for m_okPasswordCmd component and should be called instead of accessing m_okPasswordCmd field directly.//GEN-BEGIN:MVDGetBegin44
     * @return Instance for m_okPasswordCmd component
     */
    public Command get_m_okPasswordCmd() {
        if (m_okPasswordCmd == null) {//GEN-END:MVDGetBegin44
            // Insert pre-init code here
            m_okPasswordCmd = new Command("Ok", Command.OK, 1);//GEN-LINE:MVDGetInit44
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd44
        return m_okPasswordCmd;
    }//GEN-END:MVDGetEnd44
    
    /** This method returns instance for m_removePasswordCmd component and should be called instead of accessing m_removePasswordCmd field directly.//GEN-BEGIN:MVDGetBegin46
     * @return Instance for m_removePasswordCmd component
     */
    public Command get_m_removePasswordCmd() {
        if (m_removePasswordCmd == null) {//GEN-END:MVDGetBegin46
            // Insert pre-init code here
            m_removePasswordCmd = new Command("Remove", Command.ITEM, 1);//GEN-LINE:MVDGetInit46
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd46
        return m_removePasswordCmd;
    }//GEN-END:MVDGetEnd46
    
    /** This method returns instance for m_okGroupCmd component and should be called instead of accessing m_okGroupCmd field directly.//GEN-BEGIN:MVDGetBegin51
     * @return Instance for m_okGroupCmd component
     */
    public Command get_m_okGroupCmd() {
        if (m_okGroupCmd == null) {//GEN-END:MVDGetBegin51
            // Insert pre-init code here
            m_okGroupCmd = new Command("Ok", Command.OK, 1);//GEN-LINE:MVDGetInit51
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd51
        return m_okGroupCmd;
    }//GEN-END:MVDGetEnd51
    
    /** This method returns instance for exitCommand2 component and should be called instead of accessing exitCommand2 field directly.//GEN-BEGIN:MVDGetBegin53
     * @return Instance for exitCommand2 component
     */
    public Command get_exitCommand2() {
        if (exitCommand2 == null) {//GEN-END:MVDGetBegin53
            // Insert pre-init code here
            exitCommand2 = new Command("Exit", Command.EXIT, 1);//GEN-LINE:MVDGetInit53
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd53
        return exitCommand2;
    }//GEN-END:MVDGetEnd53
    
    /** This method returns instance for m_editGroupCmd component and should be called instead of accessing m_editGroupCmd field directly.//GEN-BEGIN:MVDGetBegin55
     * @return Instance for m_editGroupCmd component
     */
    public Command get_m_editGroupCmd() {
        if (m_editGroupCmd == null) {//GEN-END:MVDGetBegin55
            // Insert pre-init code here
            m_editGroupCmd = new Command("Edit", Command.ITEM, 3);//GEN-LINE:MVDGetInit55
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd55
        return m_editGroupCmd;
    }//GEN-END:MVDGetEnd55
    
    /** This method returns instance for m_passwordTextField component and should be called instead of accessing m_passwordTextField field directly.//GEN-BEGIN:MVDGetBegin57
     * @return Instance for m_passwordTextField component
     */
    public TextField get_m_passwordTextField() {
        if (m_passwordTextField == null) {//GEN-END:MVDGetBegin57
            // Insert pre-init code here
            m_passwordTextField = new TextField("Enter the main password:", "", 10, TextField.ANY | TextField.PASSWORD);//GEN-LINE:MVDGetInit57
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd57
        return m_passwordTextField;
    }//GEN-END:MVDGetEnd57
    
    /** This method returns instance for m_lockImage component and should be called instead of accessing m_lockImage field directly.//GEN-BEGIN:MVDGetBegin59
     * @return Instance for m_lockImage component
     */
    public Image get_m_lockImage() {
        if (m_lockImage == null) {//GEN-END:MVDGetBegin59
            // Insert pre-init code here
            try {//GEN-BEGIN:MVDGetInit59
                m_lockImage = Image.createImage("/icons/lock.png");
            } catch (java.io.IOException exception) {
            }//GEN-END:MVDGetInit59
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd59
        return m_lockImage;
    }//GEN-END:MVDGetEnd59
    
    /** This method returns instance for aboutForm component and should be called instead of accessing aboutForm field directly.//GEN-BEGIN:MVDGetBegin64
     * @return Instance for aboutForm component
     */
    public Form get_aboutForm() {
        if (aboutForm == null) {//GEN-END:MVDGetBegin64
            // Insert pre-init code here
            aboutForm = new Form("About", new Item[] {//GEN-BEGIN:MVDGetInit64
                get_titleStringItem(),
                get_licenseStringItem()
            });
            aboutForm.addCommand(get_backCommand1());
            aboutForm.setCommandListener(this);//GEN-END:MVDGetInit64
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd64
        return aboutForm;
    }//GEN-END:MVDGetEnd64
    
    /** This method returns instance for backCommand1 component and should be called instead of accessing backCommand1 field directly.//GEN-BEGIN:MVDGetBegin65
     * @return Instance for backCommand1 component
     */
    public Command get_backCommand1() {
        if (backCommand1 == null) {//GEN-END:MVDGetBegin65
            // Insert pre-init code here
            backCommand1 = new Command("Back", Command.BACK, 1);//GEN-LINE:MVDGetInit65
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd65
        return backCommand1;
    }//GEN-END:MVDGetEnd65
    
    /** This method returns instance for titleStringItem component and should be called instead of accessing titleStringItem field directly.//GEN-BEGIN:MVDGetBegin67
     * @return Instance for titleStringItem component
     */
    public StringItem get_titleStringItem() {
        if (titleStringItem == null) {//GEN-END:MVDGetBegin67
            // Insert pre-init code here
            titleStringItem = new StringItem("Saferoom v0.4\n", "Copyright (C) 2006\nTommi Laukkanen\nwww.substanceofcode.com\n");//GEN-LINE:MVDGetInit67
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd67
        return titleStringItem;
    }//GEN-END:MVDGetEnd67
    
    
    /** This method returns instance for licenseStringItem component and should be called instead of accessing licenseStringItem field directly.//GEN-BEGIN:MVDGetBegin69
     * @return Instance for licenseStringItem component
     */
    public StringItem get_licenseStringItem() {
        if (licenseStringItem == null) {//GEN-END:MVDGetBegin69
            // Insert pre-init code here
            licenseStringItem = new StringItem("License\n", "LGPL (See www.gnu.org/copyleft/lesser.html for more information)\n");//GEN-LINE:MVDGetInit69
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd69
        return licenseStringItem;
    }//GEN-END:MVDGetEnd69
    
    /** This method returns instance for m_groupNameTextField component and should be called instead of accessing m_groupNameTextField field directly.//GEN-BEGIN:MVDGetBegin70
     * @return Instance for m_groupNameTextField component
     */
    public TextField get_m_groupNameTextField() {
        if (m_groupNameTextField == null) {//GEN-END:MVDGetBegin70
            // Insert pre-init code here
            m_groupNameTextField = new TextField("Group name:\n", null, 32, TextField.ANY);//GEN-LINE:MVDGetInit70
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd70
        return m_groupNameTextField;
    }//GEN-END:MVDGetEnd70
    
    /** This method returns instance for m_locationTextField component and should be called instead of accessing m_locationTextField field directly.//GEN-BEGIN:MVDGetBegin73
     * @return Instance for m_locationTextField component
     */
    public TextField get_m_locationTextField() {
        if (m_locationTextField == null) {//GEN-END:MVDGetBegin73
            // Insert pre-init code here
            m_locationTextField = new TextField("Location:\n", "", 120, TextField.ANY);//GEN-LINE:MVDGetInit73
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd73
        return m_locationTextField;
    }//GEN-END:MVDGetEnd73
    
    /** This method returns instance for m_usernameTextField component and should be called instead of accessing m_usernameTextField field directly.//GEN-BEGIN:MVDGetBegin74
     * @return Instance for m_usernameTextField component
     */
    public TextField get_m_usernameTextField() {
        if (m_usernameTextField == null) {//GEN-END:MVDGetBegin74
            // Insert pre-init code here
            m_usernameTextField = new TextField("Username:\n", null, 64, TextField.ANY);//GEN-LINE:MVDGetInit74
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd74
        return m_usernameTextField;
    }//GEN-END:MVDGetEnd74
    
    /** This method returns instance for m_passwordItemTextField component and should be called instead of accessing m_passwordItemTextField field directly.//GEN-BEGIN:MVDGetBegin75
     * @return Instance for m_passwordItemTextField component
     */
    public TextField get_m_passwordItemTextField() {
        if (m_passwordItemTextField == null) {//GEN-END:MVDGetBegin75
            // Insert pre-init code here
            m_passwordItemTextField = new TextField("Password:\n", null, 120, TextField.ANY);//GEN-LINE:MVDGetInit75
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd75
        return m_passwordItemTextField;
    }//GEN-END:MVDGetEnd75
    
    /** This method returns instance for m_descriptionTextField component and should be called instead of accessing m_descriptionTextField field directly.//GEN-BEGIN:MVDGetBegin76
     * @return Instance for m_descriptionTextField component
     */
    public TextField get_m_descriptionTextField() {
        if (m_descriptionTextField == null) {//GEN-END:MVDGetBegin76
            // Insert pre-init code here
            m_descriptionTextField = new TextField("Description:\n", null, 256, TextField.ANY);//GEN-LINE:MVDGetInit76
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd76
        return m_descriptionTextField;
    }//GEN-END:MVDGetEnd76

    /** This method returns instance for cancelCommand1 component and should be called instead of accessing cancelCommand1 field directly.//GEN-BEGIN:MVDGetBegin77
     * @return Instance for cancelCommand1 component
     */
    public Command get_cancelCommand1() {
        if (cancelCommand1 == null) {//GEN-END:MVDGetBegin77
            // Insert pre-init code here
            cancelCommand1 = new Command("Cancel", Command.CANCEL, 1);//GEN-LINE:MVDGetInit77
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd77
        return cancelCommand1;
    }//GEN-END:MVDGetEnd77

    /** This method returns instance for simpleCancellableTask component and should be called instead of accessing simpleCancellableTask field directly.//GEN-BEGIN:MVDGetBegin78
     * @return Instance for simpleCancellableTask component
     */
    public org.netbeans.microedition.util.SimpleCancellableTask get_simpleCancellableTask() {
        if (simpleCancellableTask == null) {//GEN-END:MVDGetBegin78
            // Insert pre-init code here
            simpleCancellableTask = new org.netbeans.microedition.util.SimpleCancellableTask();//GEN-BEGIN:MVDGetInit78
            simpleCancellableTask.setExecutable(new org.netbeans.microedition.util.Executable() {
                public void execute() throws Exception {
                    System.out.println("Executing task");
//throw new Exception("Not ok");
                }
            });//GEN-END:MVDGetInit78
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd78
        return simpleCancellableTask;
    }//GEN-END:MVDGetEnd78

    /** This method returns instance for m_changePasswordCmd component and should be called instead of accessing m_changePasswordCmd field directly.//GEN-BEGIN:MVDGetBegin79
     * @return Instance for m_changePasswordCmd component
     */
    public Command get_m_changePasswordCmd() {
        if (m_changePasswordCmd == null) {//GEN-END:MVDGetBegin79
            // Insert pre-init code here
            m_changePasswordCmd = new Command("Change password", Command.SCREEN, 6);//GEN-LINE:MVDGetInit79
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd79
        return m_changePasswordCmd;
    }//GEN-END:MVDGetEnd79

    /** This method returns instance for newPasswordTextBox component and should be called instead of accessing newPasswordTextBox field directly.//GEN-BEGIN:MVDGetBegin81
     * @return Instance for newPasswordTextBox component
     */
    public TextBox get_newPasswordTextBox() {
        if (newPasswordTextBox == null) {//GEN-END:MVDGetBegin81
            // Insert pre-init code here
            newPasswordTextBox = new TextBox(null, null, 120, TextField.ANY);//GEN-BEGIN:MVDGetInit81
            newPasswordTextBox.addCommand(get_okCommand1());
            newPasswordTextBox.addCommand(get_cancelCommand2());
            newPasswordTextBox.setCommandListener(this);//GEN-END:MVDGetInit81
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd81
        return newPasswordTextBox;
    }//GEN-END:MVDGetEnd81

    /** This method returns instance for okCommand1 component and should be called instead of accessing okCommand1 field directly.//GEN-BEGIN:MVDGetBegin82
     * @return Instance for okCommand1 component
     */
    public Command get_okCommand1() {
        if (okCommand1 == null) {//GEN-END:MVDGetBegin82
            // Insert pre-init code here
            okCommand1 = new Command("Ok", Command.OK, 1);//GEN-LINE:MVDGetInit82
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd82
        return okCommand1;
    }//GEN-END:MVDGetEnd82

    /** This method returns instance for cancelCommand2 component and should be called instead of accessing cancelCommand2 field directly.//GEN-BEGIN:MVDGetBegin84
     * @return Instance for cancelCommand2 component
     */
    public Command get_cancelCommand2() {
        if (cancelCommand2 == null) {//GEN-END:MVDGetBegin84
            // Insert pre-init code here
            cancelCommand2 = new Command("Cancel", Command.CANCEL, 1);//GEN-LINE:MVDGetInit84
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd84
        return cancelCommand2;
    }//GEN-END:MVDGetEnd84

    /** This method returns instance for cancelCommand3 component and should be called instead of accessing cancelCommand3 field directly.//GEN-BEGIN:MVDGetBegin86
     * @return Instance for cancelCommand3 component
     */
    public Command get_cancelCommand3() {
        if (cancelCommand3 == null) {//GEN-END:MVDGetBegin86
            // Insert pre-init code here
            cancelCommand3 = new Command("Cancel", Command.CANCEL, 1);//GEN-LINE:MVDGetInit86
            // Insert post-init code here
        }//GEN-BEGIN:MVDGetEnd86
        return cancelCommand3;
    }//GEN-END:MVDGetEnd86
    
    public void startApp() {
    }
    
    public void pauseApp() {
    }
    
    public void destroyApp(boolean unconditional) {
    }
    
}
